/*
 * Copyright (c) 2005-2007 Extreme! Lab, Indiana University. All rights reserved.
 *
 * This software is open source. See the bottom of this file for the license.
 *
 * $Id: Port.java,v 1.7 2007/05/21 17:20:10 sshirasu Exp $
 */

package edu.indiana.extreme.xbaya.graph;

import java.util.Collection;

import edu.indiana.extreme.xbaya.component.ComponentPort;
import edu.indiana.extreme.xbaya.graph.gui.PortGUI;

/**
 * @author Satoshi Shirasuna
 */
public interface Port extends GraphPiece {

    /**
     * Kinds of ports
     */
    public enum Kind {
        /**
         * Output port
         */
        DATA_OUT,

        /**
         * Input port
         */
        DATA_IN,

        /**
         * CONTROL_OUT
         */
        CONTROL_OUT,

        /**
         * CONTROL_IN
         */
        CONTROL_IN,

        /**
         * EPR
         */
        EPR,
    }

    /**
     * Returns the gui
     * 
     * @return the gui
     */
    public PortGUI getGUI();

    /**
     * Returns an ID that can be used to distinguish a port, and also can be
     * used as a variable name in scripts
     * 
     * @return the ID
     */
    public String getID();

    /**
     * @return the name
     */
    public String getName();

    /**
     * @param name
     */
    public void setName(String name);

    /**
     * @return the node that this port belongs to
     */
    public Node getNode();

    /**
     * Returns the Collection of Edges that this Port is connected.
     * 
     * @return The Collection of Edges; an empty Collection if there is no
     *         edges.
     */
    public Collection<? extends Edge> getEdges();

    /**
     * Returns the Edge with a specified index.
     * 
     * @param index
     *            The specified index
     * @return The Edge with the specified index.
     */
    public Edge getEdge(int index);

    /**
     * Returns the kind of the port.
     * 
     * @return The kind of the port.
     */
    public Kind getKind();

    /**
     * Returns the Collection of Ports that this Port is connected from.
     * 
     * @return The Collection of Ports; an empty Collection if there are no
     *         Ports.
     */
    public Collection<Port> getFromPorts();

    /**
     * Returns the Port that this port is connected from.
     * 
     * @return The Port; null if there is no Port.
     */
    public Port getFromPort();

    /**
     * Returns the Collection of Nodes that this Port is connected from.
     * 
     * @return The Collection of Nodes if any; an empty Collection if there are
     *         no Nodes.
     */
    public Collection<Node> getFromNodes();

    /**
     * Returns the Nord that this port is connected from.
     * 
     * @return The Nort; null if there is no Nort.
     */
    public Node getFromNode();

    /**
     * Returns the Collection of Ports this Port is connected to.
     * 
     * @return The Collection of Ports if any; an epty Collection if there are
     *         no Ports.
     */
    public Collection<Port> getToPorts();

    /**
     * Returns the Collection of Nodes that this Port is connected to.
     * 
     * @return The Collection of Nodes if any; an empty Collection if there are
     *         no Nodes.
     */
    public Collection<Node> getToNodes();

    /**
     * Sets a ComponentPort.
     * 
     * @param componentPort
     *            The ComponentPort to set
     */
    public void setComponentPort(ComponentPort componentPort);

    /**
     * Returns the ComponentPort.
     * 
     * @return The ComponentPort
     */
    public ComponentPort getComponentPort();

}

/*
 * Indiana University Extreme! Lab Software License, Version 1.2
 * 
 * Copyright (c) 2005-2007 The Trustees of Indiana University. All rights
 * reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1) All redistributions of source code must retain the above copyright notice,
 * the list of authors in the original source code, this list of conditions and
 * the disclaimer listed in this license;
 * 
 * 2) All redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the disclaimer listed in this license in
 * the documentation and/or other materials provided with the distribution;
 * 
 * 3) Any documentation included with all redistributions must include the
 * following acknowledgement:
 * 
 * "This product includes software developed by the Indiana University Extreme!
 * Lab. For further information please visit http://www.extreme.indiana.edu/"
 * 
 * Alternatively, this acknowledgment may appear in the software itself, and
 * wherever such third-party acknowledgments normally appear.
 * 
 * 4) The name "Indiana University" or "Indiana University Extreme! Lab" shall
 * not be used to endorse or promote products derived from this software without
 * prior written permission from Indiana University. For written permission,
 * please contact http://www.extreme.indiana.edu/.
 * 
 * 5) Products derived from this software may not use "Indiana University" name
 * nor may "Indiana University" appear in their name, without prior written
 * permission of the Indiana University.
 * 
 * Indiana University provides no reassurances that the source code provided
 * does not infringe the patent or any other intellectual property rights of any
 * other entity. Indiana University disclaims any liability to any recipient for
 * claims brought by any other entity based on infringement of intellectual
 * property rights or otherwise.
 * 
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA UNIVERSITY GIVES
 * NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS.
 * INDIANA UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS",
 * "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.
 * LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION
 * GENERATED USING SOFTWARE.
 */
